<?php

/**
 * @version    5.1.0
 * @package    Com_Gavoting
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2020 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gavoting\Site\Model;

defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\MVC\Model\ListModel;
use \Joomla\CMS\Component\ComponentHelper;
use \Joomla\CMS\User\UserHelper;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GavotingHelper;

/**
 * Methods supporting a list of Gavoting records.
 *
 * @since  1.6
 */
class NominationsModel extends ListModel
{
	/**
	 * Constructor.
	 * @param   array  $config  An optional associative array of configuration settings.
	 * @see        JController
	 * @since      1.6
	 */
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'id', 'a.id',
				'ordering', 'a.ordering',
				'state', 'a.state',
				'created_by', 'a.created_by',
				'modified_by', 'a.modified_by',
				'created_date', 'a.created_date',
				'modified_date', 'a.modified_date',
				'position_id', 'a.position_id', 'pos.pos_name', 'pos.cat_id',
				'nomination', 'a.nomination',
				'nom_name', 'a.nom_name',
				'nom_date', 'a.nom_date',
				'nom_id', 'a.nom_id', 'nom.name',
				'sec_id', 'a.sec_id', 's.name',
				'agreed', 'a.agreed',
				'agreed_date', 'a.agreed_date',
				'votes', 'a.votes',
				'comment', 'a.comment',
			);
		}

		parent::__construct($config);
	}

	/**
	 * Method to auto-populate the model state.
	 * Note. Calling getState in this method will result in recursion.
	 * @param   string  $ordering   Elements order
	 * @param   string  $direction  Order direction
	 * @return void
	 * @throws Exception
	 * @since    1.6
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		$app  = Factory::getApplication();
		$list = $app->getUserState($this->context . '.list');

		$ordering  = isset($list['filter_order'])     ? $list['filter_order']     : null;
		$direction = isset($list['filter_order_Dir']) ? $list['filter_order_Dir'] : null;

		if(empty($ordering)) {
			$ordering = $app->getUserStateFromRequest($this->context . '.filter_order', 'filter_order', $app->get('filter_order'));
			if (!in_array($ordering, $this->filter_fields)) {
				$ordering = "pos.pos_name";
			}
			$this->setState('list.ordering', $ordering);
		}
		if(empty($direction)) {
			$direction = $app->getUserStateFromRequest($this->context . '.filter_order_Dir', 'filter_order_Dir', $app->get('filter_order_Dir'));
			if (!in_array(strtoupper($direction ?? ''), array('ASC', 'DESC', ''))) {
				$direction = "ASC";
			}
			$this->setState('list.direction', $direction);
		}

		$list['limit']     = $app->getUserStateFromRequest($this->context . '.list.limit', 'limit', $app->get('list_limit'), 'uint');
		$list['start']     = $app->input->getInt('start', 0);
		$list['ordering']  = $ordering;
		$list['direction'] = $direction;
		
		$app->setUserState($this->context . '.list', $list);
		//$app->input->set('list', null);   // if you use this, list always defaults to Global

        // List state information.
        parent::populateState($ordering, $direction);

        $context = $this->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
        $this->setState('filter.search', $context);
        $status = $this->getUserStateFromRequest($this->context.'.filter.state', 'filter_state');
        $this->setState('filter.state', $status);
        $catid = $this->getUserStateFromRequest($this->context.'.filter.cat_id', 'filter_cat_id');
        $this->setState('filter.cat_id', $catid);

	}

	/**
	 * Build an SQL query to load the list data.
	 * @return   JDatabaseQuery
	 * @since    1.6
	 */
	protected function getListQuery()
	{
		$params = ComponentHelper::getParams('com_gavoting');
        $user = GavotingHelper::getSpecificUser();
        $listGPs = implode(',', $user->groups);

		// Create a new query object.
        $db    = $this->getDbo();
        $query = $db->getQuery(true);

		// Select the required fields from the table.
		$query->select( $this->getState( 'list.select', 'DISTINCT a.* ' ) );
		$query->from('#__gavoting_nominations AS a');

		// Join over the users for the checked out user
		$query->select("uc.name AS uEditor");
		$query->join("LEFT", "#__users AS uc ON uc.id=a.checked_out");

		// Join over the user field 'created_by'
		$query->select('created_by.name AS created_by');
		$query->join('LEFT', '#__users AS created_by ON created_by.id = a.created_by');

		// Join over the user field 'modified_by'
		$query->select('modified_by.name AS modified_by');
		$query->join('LEFT', '#__users AS modified_by ON modified_by.id = a.modified_by');

		// Join over the user field 'nomination'
		$query->select('a.nom_name AS nomination_name, DATE_FORMAT(n.registerDate, "%d-%b-%Y") AS joinedDate');
		$query->join('LEFT', '#__users AS n ON n.id = a.nomination');

		// Join over the user field 'nom & sec'
		$query->select('nom.name AS nominator_name, s.name AS seconder_name');
		$query->join('LEFT', '#__users AS nom ON nom.id = a.nom_id');
		$query->join('LEFT', '#__users AS s ON s.id = a.sec_id');

		// Join over the position field 'position_id'
		$query->select('pos.id as pos_id, pos.pos_name AS position_id_name, pos.cat_id');
		$query->join('LEFT', '#__gavoting_positions AS pos ON pos.id = a.position_id');

		// Only show record in this users group
		if ($params->get('restrict_users',0)) {
			if ($params->get('allow_super',0) && in_array(8,$user->groups)) {
				// no filter required
			} else {
				$query->join('LEFT', '#__user_usergroup_map AS ug ON ug.user_id = a.nomination ');
	            $query->where(' ug.group_id IN ('.$listGPs.')' );
        		if ($params->get('bundle_region',0)) {
                    $custprof = $params->get('custom_profile',1);
                    if (!$custprof) {
                        // custom field
                        $fldValue = $params->get('custom_user_fld',0);
                    } elseif ($custprof === 1) {
                        // profile
                        $up = UserHelper::getProfile($user->id);
                        $fldName = $params->get('profile_user_fld',0);
                        $fldArray = explode('.', $fldName);
                        $fldValue = $up->profile[$fldArray[1]];
                        $query->join('LEFT', '#__user_profiles AS p ON p.user_id = a.nomination AND p.profile_key = '.$db->quote($fldName).' AND p.profile_value = '.$db->quote($fldValue));
                        $query->where(' p.profile_value IS NOT NULL ' );
                    } elseif ($custprof === 2) {
                        // community builder
                        $fldValue = $params->get('custom_cb_fld',0);
                    }
        		}
			}
		}

        $status = $this->getState('filter.state');
		if (isset($status) && ($status == 0 || $status == 1 || $status == 2 || $status == -2 )) {
			$query->where('a.state = '.(int) $status);
		} elseif ($status == '*')  {
			$query->where('a.state IN (0,1,2,-2)');
		} else {
            $query->where('a.state IN (1)');
		}
		
		// Only show agreed nominations
		if (!$params->get('disp_unaccept',0)) {
            $query->where(' a.agreed = 1 ');
		}

		// Filter by category
        $catid = $this->getState('filter.cat_id');
		if ($catid) {
			$query->where('pos.cat_id = ' . (int) $catid);
        }

        // Filter by search in title
        $search = $this->getState('filter.search');
        if (!empty($search)) {
            if (stripos($search, 'id:') === 0) {
                $query->where('a.id = ' . (int) substr($search, 3));
            } else {
                $search = $db->Quote('%' . $db->escape($search, true) . '%');
				$query->where('( a.nom_name LIKE ' . $search . ' OR nom.name LIKE ' . $search . ')');
            }
        }

        // Add the list ordering clause.
        $orderCol  = $this->state->get('list.ordering', "pos.pos_name");
        $orderDirn = $this->state->get('list.direction', "ASC");

        if ($orderCol && $orderDirn) {
            $query->order($db->escape($orderCol . ' ' . $orderDirn));
        }

        return $query;
	}

	/**
	 * Method to get an array of data items
	 * @return  mixed An array of data on success, false on failure.
	 */
	public function getItems()
	{
		$items = parent::getItems();
		

		return $items;
	}

	/**
	 * Overrides the default function to check Date fields format, identified by
	 * "_dateformat" suffix, and erases the field if it's not correct.
	 * @return void
	 */
	protected function loadFormData()
	{
		$app              = Factory::getApplication();
		$filters          = $app->getUserState($this->context . '.filter', array());
		$error_dateformat = false;

		foreach ($filters as $key => $value)
		{
			if (strpos($key, '_dateformat') && !empty($value) && $this->isValidDate($value) == null)
			{
				$filters[$key]    = '';
				$error_dateformat = true;
			}
		}

		if ($error_dateformat)
		{
			$app->enqueueMessage(Text::_("COM_GAVOTING_SEARCH_FILTER_DATE_FORMAT"), "warning");
			$app->setUserState($this->context . '.filter', $filters);
		}

		return parent::loadFormData();
	}

	/**
	 * Checks if a given date is valid and in a specified format (YYYY-MM-DD)
	 * @param   string  $date  Date to be checked
	 * @return bool
	 */
	private function isValidDate($date)
	{
		$date = str_replace('/', '-', $date);
		return (date_create($date)) ? Factory::getDate($date)->format("Y-m-d") : null;
	}
}
