<?php
/**
 * @version    5.1.0
 * @package    com_gavoting
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2020 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gavoting\Site\Service;

defined('_JEXEC') or die;

use \Joomla\CMS\Component\Router\RouterViewConfiguration;
use \Joomla\CMS\Component\Router\RouterView;
use \Joomla\CMS\Component\Router\Rules\StandardRules;
use \Joomla\CMS\Component\Router\Rules\NomenuRules;
use \Joomla\CMS\Component\Router\Rules\MenuRules;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Categories\Categories;
use \Joomla\CMS\Application\SiteApplication;
use \Joomla\CMS\Categories\CategoryFactoryInterface;
use \Joomla\CMS\Categories\CategoryInterface;
use \Joomla\Database\DatabaseInterface;
use \Joomla\CMS\Menu\AbstractMenu;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GavotingHelper;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GanotificationHelper;

/**
 * Class Component Router
 */
class Router extends RouterView
{
	protected $noIDs = false;

	/**
	 * The category factory
	 * @var CategoryFactoryInterface
	 * @since  4.0.0
	 */
	private $categoryFactory;

	/**
	 * The category cache
	 * @var  array
	 * @since  4.0.0
	 */
	private $categoryCache = [];

	/**
	 * The db
	 * @var DatabaseInterface
	 * @since  4.0.0
	 */
	private $db;

	public function __construct(SiteApplication $app, AbstractMenu $menu, CategoryFactoryInterface $categoryFactory, DatabaseInterface $db)
	{
		$params = Factory::getApplication()->getParams('com_gavoting');
		$this->noIDs = (bool) $params->get('sef_ids');
		$this->categoryFactory = $categoryFactory;

		/* -------------------------------------------------------- */
		/* -------   positions Config  --------------- */
		/* -------------------------------------------------------- */
		$positions = new RouterViewConfiguration('positions');
		$this->registerView($positions);

		$position = new RouterViewConfiguration('position');
		$position->setKey('id')->setParent($positions, 'cat_id');
		$this->registerView($position);

		$positionform = new RouterViewConfiguration('positionform');
		$positionform->setKey('id');
		$this->registerView($positionform);

		/* -------------------------------------------------------- */
		/* -------   nominations Config  --------------- */
		/* -------------------------------------------------------- */
		$nominations = new RouterViewConfiguration('nominations');
		$this->registerView($nominations);

		$nomination = new RouterViewConfiguration('nomination');
		$nomination->setKey('id')->setParent($nominations);
		$this->registerView($nomination);

		$nominationform = new RouterViewConfiguration('nominationform');
		$nominationform->setKey('id');
		$this->registerView($nominationform);

		/* -------------------------------------------------------- */
		/* -------   motions Config  --------------- */
		/* -------------------------------------------------------- */
		$motions = new RouterViewConfiguration('motions');
		$this->registerView($motions);

		$motion = new RouterViewConfiguration('motion');
		$motion->setKey('id')->setParent($motions);
		$this->registerView($motion);

		$motionform = new RouterViewConfiguration('motionform');
		$motionform->setKey('id');
		$this->registerView($motionform);

		/* -------------------------------------------------------- */
		/* -------   voters Config  --------------- */
		/* -------------------------------------------------------- */
		$voters = new RouterViewConfiguration('voters');
		$this->registerView($voters);

		$voter = new RouterViewConfiguration('voter');
		$voter->setKey('id')->setParent($voters);
		$this->registerView($voter);

		$voterform = new RouterViewConfiguration('voterform');
		$voterform->setKey('id');
		$this->registerView($voterform);

		/* -------------------------------------------------------- */
		/* -------   proxyvoters Config  --------------- */
		/* -------------------------------------------------------- */
		$proxyvoters = new RouterViewConfiguration('proxyvoters');
		$this->registerView($proxyvoters);

		$proxyvoter = new RouterViewConfiguration('proxyvoter');
		$proxyvoter->setKey('id')->setParent($proxyvoters);
		$this->registerView($proxyvoter);

		$proxyvoterform = new RouterViewConfiguration('proxyvoterform');
		$proxyvoterform->setKey('id');
		$this->registerView($proxyvoterform);

		/* -------------------------------------------------------- */
		/* -------    Set the rules and construct   --------------- */
		/* -------------------------------------------------------- */
		parent::__construct($app, $menu);

		$this->attachRule(new MenuRules($this));
		$this->attachRule(new StandardRules($this));
		$this->attachRule(new NomenuRules($this));
	}

	/* ------------------------------------------------------------------------ */
	/* ---------------------   Get all the Segments     ------------------ */
	/* ------------------------------------------------------------------------ */
	/**
	 * Method to get the segment(s) for a category
	 * @param   string  $id     ID of the category to retrieve the segments for
	 * @param   array   $query  The request that is built right now
	 * @return  array|string  The segments of this item
	 */
	public function getPositionsSegment($id, $query)
	{
		$category = $this->getCategories(["access" => true])->get($id);

		if ($category) {
			$path = array_reverse($category->getPath(), true);
			$path[0] = '1:root';

			if ($this->noIDs) {
				foreach ($path as &$segment) {
					list($id, $segment) = explode(':', $segment, 2);
				}
			}

			return $path;
		}

		return array();
	}

	/**
	 * Method to get the segment(s) for an record
	 * @param   string  $id     ID of the record to retrieve the segments for
	 * @param   array   $query  The request that is built right now
	 * @return  array|string  The segments of this item
	 */
	public function getPositionSegment($id, $query)
	{
		return array((int) $id => $id);
	}

	/**
	 * Method to get the segment(s) for an record
	 * @param   string  $id     ID of the record to retrieve the segments for
	 * @param   array   $query  The request that is built right now
	 * @return  array|string  The segments of this item
	 */
	public function getPositionformSegment($id, $query)
	{
		return $this->getPositionSegment($id, $query);
	}

	/**
	 * Method to get the segment(s) for an record
	 * @param   string  $id     ID of the record to retrieve the segments for
	 * @param   array   $query  The request that is built right now
	 * @return  array|string  The segments of this item
	 */
	public function getNominationSegment($id, $query)
	{
		return array((int) $id => $id);
	}

	/**
	 * Method to get the segment(s) for an record
	 * @param   string  $id     ID of the record to retrieve the segments for
	 * @param   array   $query  The request that is built right now
	 * @return  array|string  The segments of this item
	 */
	public function getNominationformSegment($id, $query)
	{
		return $this->getNominationSegment($id, $query);
	}

	/**
	 * Method to get the segment(s) for an record
	 * @param   string  $id     ID of the record to retrieve the segments for
	 * @param   array   $query  The request that is built right now
	 * @return  array|string  The segments of this item
	 */
	public function getVoterSegment($id, $query)
	{
		return array((int) $id => $id);
	}

	/**
	 * Method to get the segment(s) for an record
	 * @param   string  $id     ID of the record to retrieve the segments for
	 * @param   array   $query  The request that is built right now
	 * @return  array|string  The segments of this item
	 */
	public function getVoterformSegment($id, $query)
	{
		return $this->getVoterSegment($id, $query);
	}

	/**
	 * Method to get the segment(s) for an record
	 * @param   string  $id     ID of the record to retrieve the segments for
	 * @param   array   $query  The request that is built right now
	 * @return  array|string  The segments of this item
	 */
	public function getMotionSegment($id, $query)
	{
		return array((int) $id => $id);
	}

	/**
	 * Method to get the segment(s) for an record
	 * @param   string  $id     ID of the record to retrieve the segments for
	 * @param   array   $query  The request that is built right now
	 * @return  array|string  The segments of this item
	 */
	public function getMotionformSegment($id, $query)
	{
		return $this->getMotionSegment($id, $query);
	}

	/**
	 * Method to get the segment(s) for an invoice
	 * @param   string  $id     ID of the invoice to retrieve the segments for
	 * @param   array   $query  The request that is built right now
	 * @return  array|string  The segments of this item
	 */
	public function getProxyvoterSegment($id, $query)
	{
		return array((int) $id => $id);
	}

	/**
	 * Method to get the segment(s) for a record
	 * @param   string  $id     ID of the record to retrieve the segments for
	 * @param   array   $query  The request that is built right now
	 * @return  array|string  The segments of this item
	 */
	public function getProxyvoterformSegment($id, $query)
	{
		return $this->getProxyvoterSegment($id, $query);
	}

	/* ------------------------------------------------------------------------ */
	/* ---------------------   Get all the ID references     ------------------ */
	/* ------------------------------------------------------------------------ */
	/**
	 * Method to get the segment(s) for an record
	 * @param   string  $segment  Segment of the record to retrieve the ID for
	 * @param   array   $query    The request that is parsed right now
	 * @return  mixed   The id of this item or false
	 */
	public function getPositionId($segment, $query)
	{
		return (int) $segment;
	}

	/**
	 * Method to get the segment(s) for an transactionform
	 * @param   string  $segment  Segment of the transactionform to retrieve the ID for
	 * @param   array   $query    The request that is parsed right now
	 * @return  mixed   The id of this item or false
	 */
	public function getPositionformId($segment, $query)
	{
		return $this->getPositionId($segment, $query);
	}

	/**
	 * Method to get the id for a category
	 * @param   string  $segment  Segment to retrieve the ID for
	 * @param   array   $query    The request that is parsed right now
	 * @return  mixed   The id of this item or false
	 */
	public function getPositionsId($segment, $query)
	{
		if (isset($query['id'])) {
			$category = $this->getCategories(["access" => true])->get($query['id']);

			if ($category) {
				foreach ($category->getChildren() as $child) {
					if ($this->noIDs) {
						if ($child->alias == $segment) {
							return $child->id;
						}
					} else {
						if ($child->id == (int) $segment) {
							return $child->id;
						}
					}
				}
			}
		}

		return false;
	}

	/**
	 * Method to get the segment(s) for an invoice
	 * @param   string  $segment  Segment of the invoice to retrieve the ID for
	 * @param   array   $query    The request that is parsed right now
	 * @return  mixed   The id of this item or false
	 */
	public function getNominationId($segment, $query)
	{
		return (int) $segment;
	}

	/**
	 * Method to get the segment(s) for an invoiceform
	 * @param   string  $segment  Segment of the invoiceform to retrieve the ID for
	 * @param   array   $query    The request that is parsed right now
	 * @return  mixed   The id of this item or false
	 */
	public function getNominationformId($segment, $query)
	{
		return $this->getNominationId($segment, $query);
	}

	/**
	 * Method to get the segment(s) for an invoice
	 * @param   string  $segment  Segment of the invoice to retrieve the ID for
	 * @param   array   $query    The request that is parsed right now
	 * @return  mixed   The id of this item or false
	 */
	public function getMotionId($segment, $query)
	{
		return (int) $segment;
	}

	/**
	 * Method to get the segment(s) for an invoiceform
	 * @param   string  $segment  Segment of the invoiceform to retrieve the ID for
	 * @param   array   $query    The request that is parsed right now
	 * @return  mixed   The id of this item or false
	 */
	public function getMotionformId($segment, $query)
	{
		return $this->getMotionId($segment, $query);
	}

	/**
	 * Method to get the segment(s) for an invoice
	 * @param   string  $segment  Segment of the invoice to retrieve the ID for
	 * @param   array   $query    The request that is parsed right now
	 * @return  mixed   The id of this item or false
	 */
	public function getVoterId($segment, $query)
	{
		return (int) $segment;
	}

	/**
	 * Method to get the segment(s) for an invoiceform
	 * @param   string  $segment  Segment of the invoiceform to retrieve the ID for
	 * @param   array   $query    The request that is parsed right now
	 * @return  mixed   The id of this item or false
	 */
	public function getVoterformId($segment, $query)
	{
		return $this->getVoterId($segment, $query);
	}

	/**
	 * Method to get the segment(s) for a record
	 * @param   string  $segment  Segment of the record to retrieve the ID for
	 * @param   array   $query    The request that is parsed right now
	 * @return  mixed   The id of this item or false
	 */
	public function getProxyvoterId($segment, $query)
	{
		return (int) $segment;
	}

	/**
	 * Method to get the segment(s) for aform
	 * @param   string  $segment  Segment of the record to retrieve the ID for
	 * @param   array   $query    The request that is parsed right now
	 * @return  mixed   The id of this item or false
	 */
	public function getProxyvoterformId($segment, $query)
	{
		return $this->getProxyvoterId($segment, $query);
	}

	/* ------------------------------------------------------------------------ */
	/* ---------------------   Get Categories     ------------------ */
	/* ------------------------------------------------------------------------ */
	/**
	 * Method to get categories from cache
	 * @param   array  $options   The options for retrieving categories
	 * @return  CategoryInterface  The object containing categories
	 * @since   4.0.1
	 */
	private function getCategories(array $options = []): CategoryInterface
	{
		$key = serialize($options);

		if (!isset($this->categoryCache[$key])) {
			$this->categoryCache[$key] = $this->categoryFactory->createCategory($options);
		}

		return $this->categoryCache[$key];
	}
}
